﻿# VARIABLE VALUE EXPLANATION
#
# Variable $chk_upd_tag
# $chk_upd_tag = 0 > check gpo
# $chk_upd_tag = 1 > update gpo
#
# Permission to ADD
# $addperm = 0 > GPO Permission OK
# $addperm = 1 > ADD Domain computers / Read permissions
# $addperm = 2 > ADD Authenticated Users / Read permissions
# $addperm = 3 > ADD Authenticated Users and Domain computers / Read permissions

function updateGPOpermissions($gponame, $gpoperm){
	switch ($gpoperm) {
		1 {
			Write-Host "$gponame : `t`tADD Domain computers / Read permissions" -ForegroundColor Blue
			Set-GPPermissions -Name $gponame -PermissionLevel GpoRead -TargetName "Domain Computers" -TargetType Group
		}
		2 {
			Write-Host "$gponame : `t`tADD Authenticated Users / Read permissions" -ForegroundColor Blue
			Set-GPPermissions -Name $gponame -PermissionLevel GpoRead -TargetName "Authenticated Users" -TargetType Group
		}
		3 {
			Write-Host "$gponame : `t`tADD Authenticated Users and Domain computers / Read permissions" -ForegroundColor Blue
			Set-GPPermissions -Name $gponame -PermissionLevel GpoRead -TargetName "Domain Computers" -TargetType Group
			Set-GPPermissions -Name $gponame -PermissionLevel GpoRead -TargetName "Authenticated Users" -TargetType Group
		}
	}
}

function processCheckGPO($gponame,$gpostatus){
	$myarray = @()
	$rows = New-Object System.Object
	$addperm = 0
	
	$perm = Get-GPPermissions $gponame -All | Select-Object -ExpandProperty Trustee -Property Permission
	switch ( $gpostatus ) {
		{($gpostatus -eq "UserSettingsDisabled") -and !($perm| ? { $_.name -match "Domain computers" })} {
			$addperm = 1
		}
		{($gpostatus -eq "ComputerSettingsDisabled") -and !($perm| ? { $_.name -match "Authenticated Users" })} {
			$addperm = 2
		}
		{($gpostatus -eq "AllSettingsEnabled") -and !($perm| ? { ($_.name -match "Authenticated Users") -and ($_.name -match "Domain computers") })} {
			$addperm = 3
		}
	}
	
	$rows | Add-Member -type NoteProperty -name GpoName -Value $gponame
	$rows | Add-Member -type NoteProperty -name GpoStatus -Value $gpostatus
	$rows | Add-Member -type NoteProperty -name AddGPOPerm -Value $addperm
	
	$gpoActions += $rows
	
	return $gpoActions
}

function listGPO {
    write-host "`n`n-----------------------------------------------------"
    write-host "Select a GPO"
    write-host "-----------------------------------------------------`n"
	
	# "Default Domain Controllers Policy" and "Default Domain Policy" have been excluded from this script. Feel free to add them if you want
	$gpolist = Get-GPO -All | select displayname,GpoStatus | ? {($_.displayname -ne "Default Domain Controllers Policy") -and ($_.displayname -ne "Default Domain Policy")}| Sort-Object @{Expression='displayname'; Ascending=$True}
	$gpoinc = $gpolist.count

	for ($j=0; $j -le ($gpolist.count -1); $j++) {
		write-host $j"." $gpolist[$j].displayname
	}
    write-host "`nA. All`n"
	write-host "----------------------------------------------------"
    write-host "`nB. Cancel/Back`n"

    write-host -nonewline "Type your choice and press Enter: `n"
    
    do {
        $choice = read-host
        $ok =  (($choice -match '[0-9]') -and ([convert]::ToInt32($choice, 10) -le ($gpolist.count -1))) -or ($choice -match "x|a|b")
        if ( -not $ok) { write-host "Invalid selection" }
    } until ( $ok )

	if ($choice -notmatch "b") {
		if ($choice -notmatch "x|a") {
			# check a single GPO
			$array = processCheckGPO $gpolist[$choice].displayname $gpolist[$choice].GpoStatus
		}
		if ($choice -match "a") {
			# check all GPO
			$array = @()
			$gpolist | % {
				$array += processCheckGPO $_.displayname $_.GpoStatus
			}
		}
		
		# Display check result
		for ($i=0; $i -le 3; $i++) { 
			if (($array | ? { $_.AddGPOPerm -eq $i }).count -ge 1){
				switch ($i) {
					0 { 	Write-Host  "`nGPO permission OK`n-----------------" 
							$color_perm = "Green"
					}
					1 { 	Write-Host "`nADD Domain computers / Read permissions required on the following GPO(s)`n------------------------------------------------------------------------" 
							$color_perm = "DarkCyan"
					}
					2 { 	Write-Host "`nADD Authenticated Users / Read permissions required on the following GPO(s)`n---------------------------------------------------------------------------" 
							$color_perm = "DarkMagenta"
					}
					3 { 	Write-Host "`nADD Authenticated Users and Domain computers / Read permissions required on the following GPO(s)`n------------------------------------------------------------------------------------------------" 
							$color_perm = "DarkRed"
					}
				}
				$array | ? { $_.AddGPOPerm -eq $i } | % {
					Write-Host $_.GpoName -ForegroundColor $color_perm
				}
			}
		}
		
		# Update GPO(s)
		if ($global:chk_upd_tag -eq 1){
			do {
				Write-Host "`nPlease confirm you want to update the GPO(s) listed above`n"
			    write-host "Y. Yes"
			    write-host "N. No`n"
			    write-host -nonewline "Type your choice and press Enter: "
		    
			    $choice = read-host

				$ok = $choice -match "^(y|Y|n|N)$"
			    if ( -not $ok) { write-host "Invalid selection" }			
			} until ( $ok )
			
			if ($choice -match "^(y|Y)$"){
				Write-Host "`n`nLaunch updates on these GPOs..."
				
				$array | ? { $_.AddGPOPerm -match "^(1|2|3)$"} | % {
					updateGPOpermissions $_.GpoName $_.AddGPOPerm
				}
			}
		}
	}
}

#Main
do {
	write-host "`n---------------------------------"
	write-host "MS16-072 GPO Checker/Updater Tool"
	write-host "---------------------------------`n"
	Write-Host "`nWhat do you want to do ?"

	do {
	    write-host "`n1. Check Domain policies ?"
	    write-host "2. Update domain policies ?`n"
	    write-host "X - Exit`n"
	    write-host -nonewline "Type your choice and press Enter: "
	    
	    $choice = read-host
		
		
	    switch ( $choice ) {
		    "1"{
				$global:chk_upd_tag = 0
				listGPO
			}
			"2"{
				$global:chk_upd_tag = 1
				listGPO
			}
		}		
		write-host ""
	    $ok = $choice -match '1|2|x|X'
	    if ( -not $ok) { write-host "Invalid selection" }
	} until ( $ok )
} until ( $choice -match "X" )
